
const filename = require('./filename')

const winston = require('winston')
const util = require('util')
const { createLogger, format, transports } = winston
const { combine, timestamp, printf } = format
const { LEVEL } = require('triple-beam')

require('winston-syslog').Syslog

// rewriting syslog transport to write "silly" to "debug" and "warn" to "warning"
class YourCustomTransport extends transports.Syslog {
  constructor (opts) {
    super(opts)
  }

  log (info, callback) {
    info.level = info.level.replace(/silly/g, 'debug')
    if (info[LEVEL] === 'warn') {
      info[LEVEL] = 'warning'
    }
    if (info[LEVEL] === 'silly') {
      info[LEVEL] = 'debug'
    }
    super.log(info, callback)
  }
}

const _ = require('lodash')

const level = process.env.loglevel || process.env.LOGLEVEL || 'info'

let logFileName = false
switch (level) {
case 'silly':
case 'debug':
  logFileName = true
  break
}

let lastLength = 30
const resetLength = _.throttle(() => {
  lastLength = 30
}, 500)
const extendedInfo = (message) => {
  let transformedMessage = message

  if (logFileName) {
    transformedMessage = `${_.padEnd(message, lastLength)} ${filename()}`
    // if(message.length > lastLength) {
    //   lastLength = message.length
    // }
    resetLength()
  }
  return transformedMessage
}

const ferrariFormat = printf(info => {
  if (typeof info.message !== 'string') {
    info.message = util.inspect(info.message, false, 10)
  }
  return `${info.level}:${_.repeat(' ', 5 - info[LEVEL].length)} ${extendedInfo(info.message)}`
})

const logger = createLogger({
  format: combine(
    format.colorize(),
    timestamp(),
    ferrariFormat
  ),
  exceptionHandlers: [
    new winston.transports.Console()
  ],
  exitOnError: true
})

if (process.env.NODE_ENV !== 'production' || logFileName) {
  logger.add(new transports.Console())
}
// const t = new YourCustomTransport({protocol: 'unix', path: '/dev/log'})
const t = new YourCustomTransport({ protocol: 'udp4', host: 'syslog' })
logger.add(t)
t.on('error', () => {
  process.exit(1)
})

logger.level = level

module.exports = logger
